"""Release [0.40.3].

Revision ID: 0.40.3
Revises: ade72effebaf
Create Date: 2023-06-19 16:52:30.844851

"""

# revision identifiers, used by Alembic.
revision = "0.40.3"
down_revision = "ade72effebaf"
branch_labels = None
depends_on = None


def upgrade() -> None:
    """Upgrade database schema and/or data, creating a new revision."""
    pass


def downgrade() -> None:
    """Downgrade database schema and/or data back to the previous revision."""
    pass
