import{j as t}from"./@radix-B1sy0Lhr.js";import{C as o,S as r}from"./StackComponentList-CLnkE6-6.js";import"./plus-DOrkJCmy.js";import"./refresh-C8BXS3Fb.js";import"./Pagination-CVYIgR9l.js";import"./index-Ci5z7q6x.js";import"./@tanstack-Dwlisomv.js";import"./@react-router-CHjLNlgw.js";import"./@reactflow-gbyyU3kq.js";import"./chevron-right-double-CbMG1dOM.js";import"./SearchField-BbwrusMV.js";import"./index-g7wOyc86.js";import"./DisplayDate-57lUNS79.js";import"./expand-full-C8p_0jQv.js";import"./SheetHeader-xumE4haD.js";import"./Tabs-CkY3GwhK.js";import"./terminal-C2tAF2XG.js";import"./chevron-down-Bl7WwQAL.js";import"./transform-S7omcvTm.js";import"./NestedCollapsible-DmryIMwk.js";import"./eye-BXcQevds.js";import"./CollapsibleCard-Aqd_AiGn.js";import"./check-circle-CG6XAdk-.js";import"./CodeSnippet-D7OZMz7I.js";import"./all-pipeline-runs-query-mPYeJsKz.js";import"./ExecutionStatus-BhDlfndt.js";import"./help-CRPfbPTO.js";import"./index-BYCBKUXk.js";import"./stack-detail-query-JsrqFo3U.js";import"./ComponentBadge-BVRb7wgm.js";import"./ComponentIcon-CWDJZO4Q.js";import"./bar-chart-square-check-_m-oYysN.js";import"./layout-GIiNvS10.js";import"./trash-D9LA4VFD.js";import"./DeleteAlertDialog-C5z9RupH.js";import"./zod-D48zuELD.js";import"./index.esm-SBF9nn_P.js";import"./Infobox-CXOIY92f.js";import"./info-CI2szPUG.js";import"./bulk-delete-Bgf_MsSj.js";function O(){return t.jsx("div",{className:"pt-5",children:t.jsx(o,{children:t.jsx(r,{})})})}export{O as default};
